% In this example we apply the DWWS estimator to denoise a simulated noisy
% Doppler signal with sample size N=512 and signal-to-noise ratio SNR=5.
%
% Hyperparamter c is set to 1/3
%
%  The program calls to:
%           bayesrule_dwws_trans.m -- Computes the posterior mean for the 
%                                     DWWS estimator by numerical 
%                                     integration using a transformation to
%                                     remove singularity. Any 0<c<1 can be 
%                                     used.
%--------------------------------------------------------------------------
clear all;
close all;
set(0, 'DefaultAxesFontSize', 16);
  
N = 512;  % sample sise
SNR = 5;  % signal-to-noise ratio
coarsest = floor(log2( log (N))) + 1;  % coarsest level of decomposition
finest_lev = log2(N)-1;  % finest level

seed = 1;  % set rn generator seed
randn('state', seed );  % initialize random number generator
signal = MakeSignal('Doppler', N);  % create Doppler signal
sigma = std(signal);
signal = signal .* SNR/sigma;  % set SNR ratio
nois = randn(N,1,1);     
data = signal + nois';  % add noise to signal

filt = MakeONFilter('Symmlet', 8);  % wavelet filter

data_wd = FWT_PO (data, coarsest, filt);  % wavelet transform
data_smowd = data_wd;
niz = data_wd(dyad(finest_lev));
sigma_hat = mad(niz,1)/0.6745;  % estimate noise
c = 1/3;

for i = coarsest:finest_lev
    a = data_wd(dyad(i));
    varr = max(0,var(a) - sigma_hat^2);
    b(i) = (varr/gamma(1+2/c))^(c/2);  % set parameter b levelwise
    a_smo = 0 .* a;
    nn=length(a);
    for  j = 1:nn
        a_smo(j) = bayesrule_dwws_trans(a(j), sigma_hat^2, c, b(i));
    end;
    data_smowd(dyad(i)) = a_smo; 
end;
    
sout = IWT_PO( data_smowd, coarsest, filt);  % inverse wavelet transfrom

plot(sout); 
hold on;
plot(signal,'r-');
axis tight;
title('Denoised Doppler signal using the DWWS estimator');




