% In this example we apply the DWWS-LPM estimator to denoise a simulated noisy
% Bumps signal with sample size N=1024 and signal-to-noise ratio SNR=3.
%
% Hyperparamter c is set to 1/3
%
%  The program calls to:
%           bayesrule_dwwslpm.m -- Computes the larger posterior mode (LPM)
%                                  for the DWWS-LPM estimator. Note that 
%                                  any rational 0<c<1 in the form of 
%                                  c=cm/cn can be used.
%--------------------------------------------------------------------------
clear all;
close all;
set(0, 'DefaultAxesFontSize', 16);
  
N = 1024;  % sample sise
SNR = 3;  % signal-to-noise ratio
coarsest = floor(log2( log (N))) + 1;  % coarsest level of decomposition
finest_lev = log2(N)-1;  % finest level

seed = 1;  % set rn generator seed
randn('state', seed );  % initialize random number generator
signal = MakeSignal('Bumps', N);  % create Bumps signal
sigma = std(signal);
signal = signal .* SNR/sigma;  % set SNR ratio
nois = randn(N,1,1);     
data = signal + nois';  % add noise to signal

filt = MakeONFilter('Daubechies', 6);  % wavelet filter

data_wd = FWT_PO (data, coarsest, filt);  % wavelet transform
data_smowd = data_wd;
niz = data_wd(dyad(finest_lev));
sigma_hat = mad(niz,1)/0.6745;  % estimate noise
c = 1/3;
cm = 1;
cn = 3;

for i = coarsest:finest_lev
    a = data_wd(dyad(i));
    varr = max(0,var(a) - sigma_hat^2);
    b(i) = (varr/gamma(1+2/c))^(c/2);  % set parameter b levelwise
    a_smo = 0 .* a;
    a_smo0 = 0 .* a;
    nn=length(a);
    for  j = 1:nn
        a_smo(j) = bayesrule_dwwslpm(a(j), sigma_hat^2, cm, cn, b(i));
    end;
    data_smowd(dyad(i)) = a_smo; 
end;
    
sout = IWT_PO( data_smowd, coarsest, filt);  % inverse wavelet transfrom

plot(sout); 
hold on;
plot(signal,'r-');
axis tight;
title('Denoised Bumps signal using the DWWS-LPM estimator');



