% This function computes the posterior mean (Bayes rule) for the DWWS 
% estimator by numerical integration. The underlying equation uses a 
% transformation to remove the singularity from the posterior in order to 
% speed up the computation. Note that in this version any real 0<c<1 can be 
% used.
%
% Input
%  x -- observed wavelet coefficient
%  sigma -- noise variance
%  b, c -- hyperparameters
%
% Output
%  br -- posterior mean, shrunk version of observed wavelet coefficient
% -------------------------------------------------------------------------

function [br]=bayesrule_dwws_trans(x, sigma, c, b)

if b<0.01 br=0;
else  
for i=1:length(x)
d = x(i);
m_1 = @(th) 1/c*exp(-th/b).*exp(-(d+th.^(1/c)).^2/(2*sigma));
m_2 = @(th) 1/c*exp(-th/b).*exp(-(d-th.^(1/c)).^2/(2*sigma));
num_1 = @(th) -1/c*th.^(1/c).*exp(-th/b).*exp(-(d+th.^(1/c)).^2/(2*sigma));
num_2 = @(th) 1/c*th.^(1/c).*exp(-th/b).*exp(-(d-th.^(1/c)).^2/(2*sigma));
if (d>25) lower=(abs(d)-25)^c; else lower=0; end;
upper = (abs(d)+25)^c;
Q2(i) = quadgk(num_1,lower,upper,'MaxIntervalCount',1000,'AbsTol',1e-12,'RelTol',1e-6)+...
        quadgk(num_2,lower,upper,'MaxIntervalCount',1000,'AbsTol',1e-12,'RelTol',1e-6);
Q4(i) = quadgk(m_1,lower,upper,'MaxIntervalCount',1000,'AbsTol',1e-12,'RelTol',1e-6)+...
        quadgk(m_2,lower,upper,'MaxIntervalCount',1000,'AbsTol',1e-12,'RelTol',1e-6);
end;
S = Q2./Q4;

br = S;
end;
