% This function computes the larger posterior mode (Bayes rule) for the 
% DWWS-LPM estimator by finding roots of a polynomial. 
% Note that any rational 0<c<1 in the form of c=cm/cn can be used.
%
% Input
%  d -- observed wavelet coefficient
%  sigma -- noise variance
%  b, cm, cn -- hyperparameters (note that c=cm/cn)
%
% Output
%  br -- larger posterior mode, shrunk/thresholded version of observed 
%  wavelet coefficient
% -------------------------------------------------------------------------


function [br]=bayesrule_dwwslpm(d, sigma, cm, cn, b)

if b==0 br=0;
else
    if d>=0
    coeff = zeros(1,2*cn+1);
    coeff(2*cn+1) = -1/sigma;
    coeff(cn+1) = d/sigma;
    coeff(cm+1) = -cm/(cn*b);
    coeff(1) = cm/cn-1;
    root = roots(reverse(coeff));
    realroots = [];
    for i=1:length(root)
        if isreal(root(i)) realroots = [realroots root(i)]; end;
    end;
    if isempty(realroots) br = 0;
    else br = max(realroots)^(cn);
    end;
    if br<0 br=0; end;
    else % if d<0
        coeff = zeros(2*cn+1,1);
        coeff(2*cn+1) = -1/sigma;
        coeff(cn+1) = -d/sigma;
        coeff(cm+1) = -cm/(cn*b);
        coeff(1) = cm/cn-1;      
        root = roots(reverse(coeff));
        realroots = [];
        for i=1:length(root)
                if isreal(root(i)) realroots = [realroots root(i)]; end;
        end;
        if isempty(realroots) br = 0;
        else br = -max(realroots)^(cn);
        end;
        if br>0 br=0; end;
    end;
end;
    


